<?php
/*
	Plugin Name: WooCommerce SuperPay Boleto Gateway
	Plugin URI: http://johnhenrique.com/woocommerce/superpay-boleto-gateway-woocommerce
	Description: Integração SuperPay com boleto bancário para WooCommerce. Siga as <a href="http://www.vibemidia.com/woocommerce-superpay-boleto">instruções de uso</a>.
	Version: 1
	Author: John-Henrique 
	Author URI: http://johnhenrique.com
	Requires at least: 3.1
	Tested up to: 3.5.1
	Text domain: WC_SuperPay
*/


/**
 * Check if WooCommerce is active
 **/
if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
    // Put your plugin code here
	
	
	
	
	/**
	 * Required functions
	if ( ! function_exists( 'woothemes_queue_update' ) )
		require_once( 'woo-includes/woo-functions.php' );
	 */
	
	/**
	 * Plugin updates
	woothemes_queue_update( plugin_basename( __FILE__ ), 'WHAT IS MY KEY?', 'WHERE IS MY ID?' );
	 */
	
	add_action('plugins_loaded', 'woocomerce_gateway_superpay', 0);
	
	
	
	
	function woocomerce_gateway_superpay_boleto() {
	 
		if ( !class_exists( 'WC_Payment_Gateway' ) ) return;
		
		load_plugin_textdomain('WC_SuperPay', false, basename( dirname( __FILE__ ) ) . '/languages' );
		
		add_filter('woocommerce_payment_gateways', 'add_superpay_gateway_boleto' );
		
		
		class WC_Gateway_SuperPay_Boleto extends WC_Payment_Gateway {
			
	
			
			public function __construct() { 
				global $woocommerce;
				
				
				$this->plugin_url	= WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__));
		        $this->id			= 'superpay-boleto';
		        $this->method_title = 'SuperPay Boleto';
				$this->icon 		= WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/superpay.png';
				$this->versao		= '1';
		        $this->has_fields 	= false;
		        
		        
		        
				// Load the form fields.
				$this->init_form_fields();
				
				// Load the settings.
				$this->init_settings();
				
				// Define user set variables
				$this->title 				= $this->settings['title'];
				$this->description 			= $this->settings['description'];
				$this->token 				= $this->settings['token'];
				$this->debug				= $this->settings['debug'];	
				$this->ambiente				= $this->settings['ambiente'];
				$this->meio_pagamento		= $this->settings['meio_pagamento'];
				$this->selected_payment_type= (is_array(get_option("woocommerce_superpay_type", array()))) ? get_option("woocommerce_superpay_type", array()) : array();
				
				
				// order prefix
				$this->prefix 		= (isset($this->settings['prefix']))?$this->settings['prefix']:'';
			
				// Logs
				if ($this->debug=='yes') $this->log = new WC_Logger();
				
	
				/**
				 * URL API
				 * http://wiki.superpay.com.br/wikiSuperPay/index.php/Criar_transa%C3%A7%C3%A3o_REST
				 */
				if( $this->ambiente == 'yes' ){
					$this->charset = 'ISO-8859-1';
					$this->liveurl = 'http://homologacao.superpay.com.br/superpay/v1/transacao/'; //versao antiga, citada na documentação
					//$this->liveurl = 'https://homologacao.superpay.com.br/superpay/v1/transacao'; // versão atual não citada na documentação
				}else{
					$this->charset = 'UTF-8';
					$this->liveurl = 'https://superpay2.superpay.com.br/checkout/v1/transacao';
				}
				
				
	
				if ( ! $this->is_valid_currency() || ! $this->are_credentials_set() )
					$this->enabled = false;
				
					
					
				// 2.0
				add_action( 'woocommerce_update_options_payment_gateways_'. $this->id, array( $this, 'process_admin_options' ) );
				add_action( 'woocommerce_api_'. strtolower( get_class( $this ) ), array( $this, 'helper_consulta_transacao' ) );
				//add_action( 'woocommerce_api_teste2', array( $this, 'helper_consulta_transacao' ) );// not working
	
				
				// Actions
				add_action('woocommerce_receipt_superpay',	array(&$this, 'receipt_page' ));
				
				
				// support to subscription plugin
				$this->supports = array(  
	               'products', 
	               'subscriptions',
	               'subscription_cancellation', 
	               'subscription_suspension', 
	               'subscription_reactivation',
	               'subscription_amount_changes',
	               'subscription_date_changes',
	               'subscription_payment_method_change'
               );
               
               
               
               
				add_action('wp_enqueue_scripts', array( $this, 'js_superpay' ) );
				add_action('wp_enqueue_scripts', array($this, 'css_superpay' ) );
		    } 
		    
		    
			/**
		     * Initialise Gateway Settings Form Fields
		     */
		    function init_form_fields() {
		    
		    	$this->form_fields = array(
					'enabled' => array(
									'title' => __( 'Enable/Disable', 'WC_SuperPay' ), 
									'type' => 'checkbox', 
									'label' => __( 'Enable SuperPay standard', 'WC_SuperPay' ), 
									'default' => 'yes'
								), 
					'title' => array(
									'title' => __( 'Title', 'WC_SuperPay' ), 
									'type' => 'text', 
									'description' => __( 'This controls the title which the user sees during checkout.', 'WC_SuperPay' ), 
									'default' => __( 'SuperPay', 'WC_SuperPay' )
								),
					'description' => array(
									'title' => __( 'Description', 'WC_SuperPay' ), 
									'type' => 'textarea', 
									'description' => __( 'This controls the description which the user sees during checkout.', 'WC_SuperPay' ), 
									'default' => __( "Pague usando seu cartão de crédito Cielo, Mastercard, Diners Club, American Express, Discover e Elo.", 'WC_SuperPay')
								),
					'token' => array(
									'title' => __( 'SuperPay Token', 'WC_SuperPay' ), 
									'type' => 'text', 
									'description' => sprintf( __( 'Please enter your SuperPay token; this is needed in order to take payment. See %sToken of security%s', 'WC_SuperPay' ), '<a href="http://johnhenrique.com/woocommerce/superpay-gateway-woocommerce" target="_blank">', '</a>' ), 
									'default' => ''
								),
					'parcelas' => array(
									'title' => __( 'Quantidade máxima de parcelas', 'WC_SuperPay' ),
									'type' => 'text', 
									'description' => sprintf(__( 'Informe o número de parcelas que o comprador pode dividir o pagamento no cartão de crédito. Veja %snúmero de parcelas%s para mais informações', 'WC_SuperPay'), '<a href="http://johnhenrique.com/woocommerce/superpay-gateway-woocommerce" target="_blank">','</a>'), 
									'default' => '5'
								),
					'meio_pagamento' => array(
									'title' => __( 'Operadora', 'wc-locaweb-cielo' ), 
									'type' => 'select', 
									'description' => __( 'Operadora de pagamentos', 'wc-locaweb-cielo' ), 
									'options' => array(
													'cielo' => 'CIELO',
													'redecard_web' => 'REDECARD',
												),
									'default' => 'cielo'
								),
								/*
					'prefix' => array(
									'title' => __( 'Order prefix', 'WC_SuperPay' ), 
									'type' => 'text', 
									'description' => __( 'Choose a prefix to orders. Eg WC this case order ID is would "WC-5126452".', 'WC_SuperPay' ), 
									'size' => 2, 
									'default' => 'WC'
								),
								*/
					'ambiente' => array(
									'title' => __( 'Enable sandbox', 'WC_SuperPay' ), 
									'type' => 'checkbox', 
									'description' => __( 'Enable sandbox (Test mode)', 'WC_SuperPay' ), 
									'default' => 'sandbox'
								),
					'debug' => array(
									'title' => __( 'Enable Log system', 'WC_SuperPay' ), 
									'type' => 'checkbox', 
									'description' => sprintf( __( 'Enable logging (<code>woocommerce/logs/%s.txt</code>)', 'WC_SuperPay' ), strtolower( $this->id ) ), 
									'default' => ''
								)
					
					);
		    
		    } // End init_form_fields()
		    
		    
			/**
			 * Admin Panel Options 
			 * - Options for bits like 'title' and availability on a country-by-country basis
			 *
			 * @since 1.0.0
			 */
			public function admin_options() {
		
		    	?>
		    	<h3><?php echo __('SuperPay Gateway', 'WC_SuperPay' ); ?></h3>
		    	<p><?php echo sprintf( __( 'SupePay é o gateway da Locaweb, antigo Gateway Locaweb. Se você utilizava o Gateway Locaweb você precisará %sler este documento%s.', 'WC_SuperPay'), '<a href="http://johnhenrique.com/woocommerce/superpay-gateway-woocommerce">', '</a>' ); ?></p>
		    	<table class="form-table">
					<?php if ( ! $this->is_valid_currency() ) : ?>
						<div class="inline error">
							<p><strong><?php _e( 'Gateway Disabled', 'WC_SuperPay' ); ?></strong>: <?php _e( 'SuperPay does not support your store\'s currency. You need to select the currency of Brazil Real.', 'WC_SuperPay' ); ?></p>
						</div>
					<?php endif; ?>
					
					<?php if ( ! $this->are_credentials_set() ) : ?>
						<div class="inline error">
							<p><strong><?php _e( 'Gateway Disabled', 'WC_SuperPay' ); ?></strong>: <?php _e( 'Você precisa informar o token da sua conta SuperPay.', 'WC_SuperPay' ); ?></p>
						</div>
					<?php endif; ?>
					
					<?php
					if ( ! $this->check_payment_method() ) :
						echo "<div class=\"inline error\">
							<p><strong>". __( 'Gateway Disabled', 'WC_SuperPay' ) ."</strong>: ". __( 'You must choose a payment method.', 'WC_SuperPay' ) ."</p>
						</div>";
					endif;
					?>
		    	<?php
		    		// Generate the HTML For the settings form.
		    		$this->generate_settings_html();
		    	?>

                <tr valign="top">
                    <th scope="row" class="titledesc"><?php _e('Payment methods', 'WC_SuperPay') ?></th>
                    <td class="forminp">
                        <?php
                        $payment_type = array( 
                        	'visa' 		=> "Visa", 
                        	'mastercard'=> "Mastercard", 
                        	'diners' 	=> "Diners Club", 
                        	'amex' 		=> "American Express (AMEX)", 
                        	'elo'		=> "ELO", 
                        	'discover' 	=> "Discover"
                        );
                        foreach( $payment_type as $payment => $rotulo ):
                            ?>
                            <input type="checkbox" name="woocommerce_superpay_type[]" value="<?php echo $payment ?>" id="<?php echo $payment ?>" 
                            <?php
				                if (in_array( $payment, $this->selected_payment_type)) {
				                    echo 'checked="checked"';
				                }
                            ?> /> <label for="<?php echo $payment; ?>"><?php echo $rotulo ?></label>
                            </br>
                        <?php endforeach; ?>
                        <p class="description"><?php _e( "You can choose what payment method is accepted" , 'WC_SuperPay' ); ?></p>
                    </td>
                </tr>
				</table><!--/.form-table-->
		    	<?php
		    } // End admin_options()
	
		    
		    
			
	        public function process_admin_options() {
	            parent::process_admin_options();
	
	            if (isset($_POST['woocommerce_superpay_type']))
	                $selected_payment_type = $_POST['woocommerce_superpay_type'];
	            else
	                $selected_payment_type = array();
	            update_option('woocommerce_superpay_type', $selected_payment_type);
	        }
	        
	        
		    
			/**
			 * Check if SuperPay can be used with the store's currency.
			 * For now only work with Real of Brazil, but...
			 *
			 * @since 1.0
			 */
			function is_valid_currency() {
				if ( !in_array( get_option( 'woocommerce_currency' ), array( 'BRL' ) ) )
					return false;
				else
					return true;
			}
			


			/**
			 * Check if payment method was choosed
			 *
			 * @since 2.5
			 */
			function check_payment_method() {
				if ( empty( $this->selected_payment_type ) )
					return false;
				else
					return true;
			}
	
			
			/**
			 * Check if SuperPay Credentials are set
			 *
			 * @since 1.0
			 */
			function are_credentials_set() {
				if( empty( $this->token ) )
					return false;
				else
					return true;
			}
		    
		    
		    /**
			 * There are no payment fields for SuperPay, but we want to show the description if set.
			 * When this->has_fields is true, need add payment code here
			 **/
		    function payment_fields() {
		    	if ($this->description) echo wpautop(wptexturize($this->description));
		    	
		    	echo $this->html_payment();
		    }
		    
		    

			
			/**
			 * Se encarrega de remover os dados confidenciais 
			 * como número do cartão e código de segurança
			 *
			 * @param String $str
			 * @return String
			 */
			protected function privacidade( $str = '' ){
				$procura = array(
					'/"cartao_numero"\:"(.?)+",/',
					'/"cartao_cvv"\:"(.?)+",/'
				);
				$troca = array(
					'"cartao_numero":"XXXXXXXXXXXXX",',
					'"cartao_cvv":"XXX",'
				);
				//return preg_replace( $procura, $troca, $str );
				return $str;
			}
	
	
		    
		    
		    public function json2( $order ){
		    	
		    	$campo = $_POST;
		    	
		    	$int_randon = rand(7,50);
	
		    	/**
		    	 * Campos marcados com * são considerados 
		    	 * sigilosos e só devem ser solicitados 
		    	 * na loja no modo Buy Page Loja ou 
		    	 * Redecard Komerci Web Service
		    	 */
				$arr_superPay = array();
				
				$arr_superPay['token'] = $this->token;
				
				$url_retorno = $this->get_return_url( $order );
				
		    	// What is status in this case?
		    	if ($this->debug=='yes') $this->log->add( $this->id, "URL retorno: ". $url_retorno );
		    	
				
				$arr_superPay['transacao'] = array(
					'url_retorno' 		=> $url_retorno, 
					'capturar' 			=> 'true', 
				);
				$arr_superPay['transacao']['pedido'] = array(
					'numero' 			=> $order->id,
					'total' 			=> number_format( $order->order_total, 2, '.', '' ),
					'moeda'				=> 'real', 
					'descricao'			=> get_option( 'blogname' ) ." ". sprintf( __( "order #%s", 'WC_SuperPay' ), $this->prefix . $order->id )
				);
				$arr_superPay['pagamento'] = array(
					'bandeira'			=> $campo['bandeira'], 
					'meio_pagamento'	=> $this->meio_pagamento, // operadora de pagamento CIELO / REDECARD
					'cartao_numero'		=> $campo['numero_cartao'], // *
					'nome_titular_cartao' 	=> $campo['nome_portador_cartao'], 
					'cartao_cvv'		=> $campo['codigo_seguranca_cartao'], // *
					'cartao_validade' 	=> $campo['mes'] . $campo['ano'], // *
					'parcelas'			=> $campo['parcelas'], 
					'tipo_operacao' 	=> $this->cartao_avista_aprazo()
				);
				$arr_superPay['comprador'] = array(
					'nome' 				=> $campo['nome_portador_cartao'], 
					'documento'			=> $campo['cpf_portador_cartao'], 
					'endereco'			=> $campo['endereco'], 
					'complemento'		=> $campo['complemento'], 
					'numero'			=> $campo['numero'],
					'cep'				=> str_replace( '-', '', $campo['billing_postcode'] ), 
					'bairro'			=> $campo['bairro'], 
					'cidade'			=> $campo['billing_city'],
					'estado'			=> $campo['billing_state']
				);
				
				$arr_superPay = json_decode('{ 
				"token":"'. $this->token .'",
				"transacao":{
				"url_retorno":"'. $url_retorno .'",
				"capturar":"true",
				"pedido":{
				"numero":"'. $this->prefix . $order->id .'",
				"total":"'. number_format( $order->order_total, 2, '.', '' ) .'",
				"moeda":"real",
				"descricao":"WooCommerce order #'. $this->prefix . $order->id .'"
				},
				"pagamento":{
				"bandeira":"'. $campo['bandeira'] .'",
				"meio_pagamento":"'. $this->meio_pagamento .'",
				"cartao_numero":"'. $campo['numero_cartao'] .'",
				"nome_titular_cartao":"'. $campo['nome_portador_cartao'] .'",
				"cartao_cvv":"'. $campo['codigo_seguranca_cartao'] .'", 
				"cartao_validade":"'. $campo['mes'] . $campo['ano'] .'",
				"parcelas":"'. $campo['parcelas'] .'",
				"tipo_operacao":"'. $this->cartao_avista_aprazo() .'"
				},
				"comprador":{
				"nome":"'. $campo['nome_portador_cartao'] .'",
				"documento":"'. $campo['cpf_portador_cartao'] .'",
				"endereco":"'. $campo['endereco'] .'",
				"complemento":"'. $campo['complemento'] .'",
				"numero":"'. $campo['numero'] .'",
				"cep":"'. str_replace( '-', '', $campo['billing_postcode'] ) .'",
				"bairro":"'. $campo['bairro'] .'",
				"cidade":"'. $campo['billing_city'] .'",
				"estado":"'. $campo['billing_state'] .'"
				}
				}
				}');

				return $arr_superPay;
		    }
		    
	
			
			
			/**
			 * Informa se a compra será com cartão 
			 * de crédito a vista ou a prazo
			 *
			 * @see http://docs.gatewaylocaweb.com.br/referencia-da-api/criar-transacao#TipoOperacoes%7Ctable
			 * @return String (credito_a_vista, credito_a_prazo ou parcelado_administradora)
			 */
			protected function cartao_avista_aprazo(){
				if( $_POST['parcelas'] != 1 ){
					
					if( $this->parcelado_por_loja == 'yes' ){
						
						if ($this->debug=='yes') $this->log->add(  $this->id,  "Pagamento parcelado pela loja" );
						return 'parcelado_loja';
					}else{
						
						if ($this->debug=='yes') $this->log->add(  $this->id,  "Pagamento parcelado pela administradora" );
						return 'parcelado_administradora';
					}
				}else{
					if ($this->debug=='yes') $this->log->add(  $this->id,  "Pagamento à vista" );
					return 'credito_a_vista';
				}
			}
		    
		    
		    
		    /**
		     * Helper para auxiliar na identificação do status 
		     * do pedido, neste caso, iremos verificar e retornar 
		     * uma mensagem para o histórico do pedido, além de 
		     * alterar o status atual do pedido
		     *
		     * @param String $status
		     * @param Object $order
		     * @return Array
		     */
		    public function status_helper( $status, $order ){
		    	global $woocommerce;
		    	
		    	
		    	switch ( $status ) {
		    		case 'falha_na_operadora':
		    			
						$arrStatus['status'] = "failed"; // falha no pagamento
						$arrStatus['log'] = __( 'Houve um erro durante a tentativa de pagamento. Esta transação não foi paga, você não será cobrado.', 'WC_SuperPay' ); // falha no pagamento
						
		    			break;
		    		
		    		case 'Aguardando Pagamento':
		    		case 'aguardando pagamento':
		    			// A transação foi criada, mas o pagamento ainda não foi efetuado.
		    			
						$arrStatus['status'] = "on-hold"; // aguardando pagamento
						$arrStatus['log'] = __( 'Payment pending, Waiting payment confirmation', 'WC_SuperPay' ); // aguardando pagamento
						
						// add note to control
						$order->add_order_note( $arrStatus['log'] );
						//$order->reduce_order_stock();
						$woocommerce->cart->empty_cart();
	
						break;
		    	
		    		case 'Pago':
		    		case 'pago':
		    		case 'paga':
		    			// A transação foi criada e o pagamento foi efetuado com sucesso.
		    			
						// Reduz estoque
						$order->reduce_order_stock();
						
			    		// change payment status
			    		$order->payment_complete();
			    		
			    		
						$woocommerce->cart->empty_cart();
			    		
			    		// remove a marcação que informa para realizar consulta no servidor
			    		delete_post_meta( $order->id, '_superpay_api' );
		    			
		    			break;
		    	
		    		case 'Negado':
		    		case 'negado':
		    		case 'negada':
		    			// A transação foi criada, mas houve falha no pagamento. Ex: cartão de crédito sem limite suficiente.
		    			
						$arrStatus['status'] = "failed"; // aguardando pagamento
						$arrStatus['log'] = __( 'Payment denied. Your payment is not authorized.', 'WC_SuperPay' ); // aguardando pagamento
						
						
						// add note to control
						$order->add_order_note( $arrStatus['log'] );
						
						
						// informa na tela do checkout sobre o problema com o pagamento
						$this->add_error( $arrStatus['log'], 'error' );
		    			
		    			break;
		    	
		    		case 'Cancelado':
		    		case 'cancelado':
		    		case 'cancelada':
		    			// O pagamento efetuado foi cancelado (estornado) pelo lojista.
		    			
						$arrStatus['status'] = "failed"; // aguardando pagamento
						$arrStatus['log'] = __( 'Order canceled or reversed manually', 'WC_SuperPay' ); // aguardando pagamento
						
			    		// cancel this order 
			    		$order->cancel_order( $order->id );
			    		
			    		// remove a marcação que informa para realizar consulta no servidor
			    		delete_post_meta( $order->id, '_superpay_api' );
		    			
		    			break;
		    	
		    		default: // status não identificado
		    			
						$arrStatus['status'] = "on-hold"; // Falha, status não identificado
						$arrStatus['log'] = sprintf( __( 'Status not identified: %s', 'WC_SuperPay'), $status ); // aguardando pagamento
						
						// add note to control
						$order->add_order_note( $arrStatus['log'] );
						
						$woocommerce->cart->empty_cart();
						
		    			break;
		    	}
		    	
				
				$order->update_status( $arrStatus['status'], $arrStatus['log'] );
		    	
		    	// What is status in this case?
		    	if ($this->debug=='yes') $this->log->add( $this->id, "Status \n Status found: ". $arrStatus['status'] ."\n Status source: ". $status );
				
				return $arrStatus;
		    }
		    
		    
		    /**
		     * Permite consultar as transações no servidor 
		     * do SuperPay, lá não existe retorno automático 
		     * sendo assim, teremos que ativar uma tarefa 
		     * cron para realizar esta tarefa
		     * 
		     * Alternativamente podemos utilizar o wp-cron 
		     * porém ele depende de acessos reais
		     * 
		     * É necessário informar o ID da transação 
		     * e o token de acesso no SuperPay
		     *
		     */
		    public function helper_consulta_transacao(){
		    	global $woocommerce;
		    	
		    	
		    	/**
		    	 * Aparentemente o ID da transação é diferente do ID do pedido 
		    	 * se for podemos adicionar um campo personalizado no padrão 
		    	 * ID_TRANSACAO-ID_PEDIDO 
		    	 * E consultar estes itens, desta forma saberemos quais 
		    	 * pedidos ainda precisam ser verificados no servidor
		    	 */
		    	query_posts(array(
		    		'post_type' 	=> 'shop_order', 
		    		'post_status' 	=> 'publish', 
		    		'posts_per_page'=> 10 // limitando os resultados (deve ter outro modo)
		    		//'meta_key' 		=> '_verified', 
		    		//'meta_value' 	=> 'yes', 
		    		//'meta_compare' 	=> '!='
		    	));
		    	
		    	// The Loop
				while ( have_posts() ) : the_post();
				    
				    $order = new WC_Order( get_the_ID() );
				    
			    	if ($this->debug=='yes') $this->log->add( $this->id, "Helper Consulta Transação: ". get_the_ID() );
		
			    	
			    	$url_api = $this->liveurl ."/". $order->id ."?token=". $this->token;
			    	
		    		if ($this->debug=='yes') $this->log->add( $this->id, "SuperPay URL API: ". $url_api );
			    	
			    	// adicionar o WP_http
			    	$superpay_api = wp_remote_get( $url_api );
			    	
			    	unset( $superpay_api['headers'] );
			    	unset( $superpay_api['cookies'] );
			    	unset( $superpay_api['filename'] );
			    	
		    		if ($this->debug=='yes') $this->log->add( $this->id, "SuperPay API response: ". print_r( $superpay_api, true ) );
		    		
			    	
					if( is_wp_error( $superpay_api ) ){
						
						$error_message = $superpay_api->get_error_message();
						$this->add_error( sprintf( __("There was an error: %s", 'WC_SuperPay' ), $error_message ), 'error' );
						
						
			    		if ($this->debug=='yes') $this->log->add( $this->id, "API returned error: ". $error_message );
			    		
					}else{
						
						$superpay_api2 = json_decode( $superpay_api['body'] );
						
				    	// acessando variaveis
				    	$superpay_order_id 		= $superpay_api2->transacao->id;
				    	$superpay_order_status 	= $superpay_api2->transacao->status; // paga para concluido e negada para falha
				    	$superpay_order_response= $superpay_api['response']['code'] .' '. $superpay_api['response']['message'];
				    	
				    	
			    		if ($this->debug=='yes') $this->log->add( $this->id, "SuperPay Order Status: ". $superpay_order_status );
			    		if ($this->debug=='yes') $this->log->add( $this->id, "SuperPay Order ID: ". $superpay_order_id );
			    		if ($this->debug=='yes') $this->log->add( $this->id, "SuperPay Response: ". $superpay_order_response );
			    		
				    	$this->status_helper( $superpay_order_status, $order );
					}
		    	
		    	
				    
				endwhile;
				
				//echo $GLOBALS['wp_query']->request;
				
				// Reset Query
				wp_reset_query();
		    }
		    
		    
	
			
			/**
			 * Process the payment and return the result
			 * checkout
			 * checkout page
			 * 
			 **/
			function process_payment( $order_id = 0 ) {
				global $woocommerce;
				
				$order = new WC_Order( $order_id );
	
				unset( $order->order_custom_fields );
				
				
				if($this->debug=='yes') $this->log->add( $this->id, 'WooCommerce '. WOOCOMMERCE_VERSION );
				if($this->debug=='yes') $this->log->add( $this->id, 'WooCommerce '. get_class( $this ) .' '. $this->versao );
				if($this->debug=='yes') $this->log->add( $this->id, 'Ambiente de testes: '. $this->ambiente );
				if($this->debug=='yes') $this->log->add( $this->id, 'Charset: '. $this->charset );
				
				
				/**
				 * Todo o processo de validação foi realizado 
				 * na tela de checkout/opções de pagamento
				 * aqui iremos realizar a chamada da transação 
				 * 
				 * Array (sem json)
				 */
				$obj_json = json_encode( $this->json2( $order ) );
				
				$dados = json_decode( $this->privacidade( $obj_json ) );
				unset( $dados->transacao->pagamento->cartao_numero );
				unset( $dados->transacao->pagamento->cartao_cvv );
				unset( $dados->transacao->pagamento->cartao_validade );
				unset( $dados->transacao->comprador->documento );
				if($this->debug=='yes') $this->log->add( $this->id, 'Array: '. print_r( $dados, true ) );
				if($this->debug=='yes') $this->log->add( $this->id, 'Objeto JSon: '. print_r( json_encode( $this->privacidade( $obj_json ) ), true ) );
				if($this->debug=='yes') $this->log->add( $this->id, 'URL API: '. $this->liveurl );
				if($this->debug=='yes') $this->log->add( $this->id, 'Server encoding: '. print_r( iconv_get_encoding('all'), true ) );
					
				/*
				$superpay_api = wp_remote_post( $this->liveurl, array(
	        		'method' 	=> 'POST', 
	        		'headers' 	=> array( 
	        						'Content-Type: application/x-www-form-urlencoded', //; charset='. $this->charset, 
	        						//'Media-Type: application/x-www-form-urlencoded', 
	        						//'sslversion' => 3,
	        		), 
	        		'httpversion' => '1.0', 
	        		'body' 		=> $obj_json,  
	        		'sslverify' => true, 
	        		'timeout' 	=> 45, 
	        		'sslversion' => 3 // testes
				));
				*/
				
				//print_r( $obj_json );
				
				$curl = curl_init( $this->liveurl );
				curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
				curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
				//curl_setopt($curl, CURLOPT_HTTPHEADER, array("Content-Type: application/json; charset=". $this->charset ));
				curl_setopt($curl, CURLOPT_HTTPHEADER, array("Content-Type: application/x-www-form-urlencoded; charset=". $this->charset ));
				curl_setopt($curl, CURLOPT_POSTFIELDS, $obj_json);
				curl_setopt($curl, CURLOPT_SSLVERSION, 3);
				//curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 45);
				curl_setopt($curl, CURLOPT_TIMEOUT, 180);
				$curl_superpay_api = curl_exec($curl);
				
				
				
				$superpay_api = json_decode( $curl_superpay_api );
				
				
				//if( is_wp_error( $superpay_api ) ){
				if( curl_exec( $curl ) === false ){
					
					//$sp_error = $superpay_api->get_error_message();
					
					$sp_error = curl_error( $superpay_api );
					
					
					$http_status = $this->http_status();
					
					
					$this->add_error( " Erro: ". print_r( $sp_error, true ) . $http_status, 'error' );
					if($this->debug=='yes') $this->log->add( $this->id, 'Houve um erro na requisição: '. $sp_error  .' - HTTP: '. $http_status);
					
					return;
				}else{
					
					if($this->debug=='yes') $this->log->add( $this->id, 'Requisição realizada: '. print_r( $superpay_api, true ) );
					
					
					
					// removendo inutilidades
					if( isset( $superpay_api['headers'] ) ){ unset( $superpay_api['headers'] ); }
			    	if( isset( $superpay_api['cookies'] ) ){ unset( $superpay_api['cookies'] ); }
			    	if( isset( $superpay_api['filename'] ) ){ unset( $superpay_api['filename'] ); }
			    	
					
			    	// acessando variaveis
			    	$superpay_order_id 		= $superpay_api->transacao->id;
			    	$superpay_order_status 	= $superpay_api->transacao->status; // paga para concluido e negada para falha
			    	$superpay_order_response= $superpay_api->transacao->id .' '. $superpay_api->transacao->erro;
			    	
			    	
				    	
		    		if ($this->debug=='yes') $this->log->add( $this->id, "SuperPay Order Status: ". $superpay_order_status );
		    		if ($this->debug=='yes') $this->log->add( $this->id, "SuperPay Order ID: ". $superpay_order_id );
		    		if ($this->debug=='yes') $this->log->add( $this->id, "SuperPay Response: ". $superpay_order_response );
		    		if ($this->debug=='yes') $this->log->add( $this->id, print_r( $superpay_api, true ) );
		    		
				    	
					/**
					 * Quando o retorno for um código 200 significa 
					 * que a requisição foi recebida corretamente 
					 * então, temos dados para trabalhar
					 */
					if( $superpay_api->response->code == 200 || $superpay_api->response->code == 201 ){
						
				    	
			    		
				    	$this->status_helper( $superpay_order_status, $order );
				    	

							// clean cart
							$woocommerce->cart->empty_cart();
							
							
							// Empty awaiting payment session
							// 2.0
							unset( $woocommerce->session->order_awaiting_payment );
							unset( $_POST ); // limpando todos os campos
							
							
							return array(
								'result' 	=> 'success',
								'redirect'	=> $this->get_return_url( $order )
							);
							
				    		
				    		// transação foi paga anteriormente
				    		if( $superpay_order_status == 'ja_paga' ){
						    			
				    			$link = '<a href="'. get_permalink( get_option('woocommerce_myaccount_page_id') ) .'/view-order/'. $order->id .'">veja detalhes clicando aqui</a>';
				    			
								// mensagem no checkout
								$this->add_error(__('Payment error:', 'WC_SuperPay') ." Esta compra já foi paga anteriormente, ". $link, 'error' );
								
								// alterando status do pagamento
								$order->payment_complete();
				    			
				    			return;
				    		}
				    	//}
				    	
				    	
						
					}else{
						/**
						 * This payment can be denied or aproved 
						 * so can't use 'payment_complete' or 
						 * 'empty_cart' here
						 */
						
						
						
				    	$status_helper = $this->status_helper( $superpay_order_status, $order );
						
						/*
						$superpay_api_2 = ( !empty( $superpay_api->transacao->erro ) )? ' - '. print_r( $superpay_api->transacao->erro, true ):'';
						
						// algum erro ocorreu
						$superpay_api_error = $superpay_api->transacao->status . $superpay_api_2;
						
						
						// log
						if($this->debug=='yes') $this->log->add( $this->id, 'Houve um erro: '. $superpay_api_error );
						*/
						
						
						//$order->update_status( 'failed', __('Este pagamento já foi realizado anteriormente', 'WC_SuperPay'));
						
						// mensagem no checkout
						//$this->add_error(__('Payment error:', 'WC_SuperPay') .' Este pagamento já foi realizado anteriormente, veja detalhes clicando em "<a href="'. get_permalink( get_option('woocommerce_myaccount_page_id') ) .'/view-order/'. $order->id .'">Minha conta</a>".', 'error' );
						
						
						return;
					}
					
				}
				/*
				$this->add_error(__('Payment error:', 'WC_SuperPay') . "Simulando um erro do gateway", 'error' );
				return;
				*/
			}
			
			
	
			
			/**
			 * Checkout -> Pay
			 * Payment page, redirect to gateway
			 **/
			function receipt_page( $order_id ) {
				echo '<p>'.__('Thank you for your order, please click the button below to pay with SuperPay.', 'WC_SuperPay').'</p>';
				
				if($this->debug=='yes') $this->log->add( $this->id, 'Receipt page' );
			}
			
			
			
			public function html_payment(){
	
				
				/**
				 * Calculando os próximos 6 anos
				 * isto evita que seja necessário 
				 * atualizar o plugin para continuar 
				 * utilizando-o no futuro.
				 */
				$ano = '';
				$anoAtual = date( 'Y' );
				for( $i=$anoAtual; $i <= ($anoAtual+8); $i++ ){
					$ano_selecionado = (isset( $_POST['ano'] ) && $_POST['ano'] == $i )?'checked':'';
					$ano .= "<option value='". $i ."' ". $ano_selecionado .">". $i ."</option>";
				}
				
				
				/**
				 * Calcula a quantidade de parcelas 
				 * que pode-se escolher
				 */
				$parcelas = '';
				for( $i=1; $i <= $this->parcelas; $i++ ){
					$parcela_selecionada = (isset( $_POST['parcelas'] ) && $_POST['parcelas'] == $i )?'checked':'';
					$parcelas .= "<option value='". $i ."'". $parcela_selecionada .">". $i ."</option>";
				}
				
				
				$imagens = WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images';
				
				
				
				
				$nome_portador 	= (isset( $_POST['nome_portador_cartao'] ) && $_POST['nome_portador_cartao'] != '' )? $_POST['nome_portador_cartao']:'';
				$numero_cartao 	= (isset( $_POST['numero_cartao'] ) && $_POST['numero_cartao'] != '' )? $_POST['numero_cartao']:'';
				$codigo_seguranca_cartao 	= (isset( $_POST['codigo_seguranca_cartao'] ) && $_POST['codigo_seguranca_cartao'] != '' )? $_POST['codigo_seguranca_cartao']:'';
				
				$visa 			= (isset( $_POST['bandeira'] ) && $_POST['bandeira'] == 'visa' )? 'checked="checked"':'';
				$mastercard		= (isset( $_POST['bandeira'] ) && $_POST['bandeira'] == 'mastercard' )? 'checked="checked"':'';
				$diners			= (isset( $_POST['bandeira'] ) && $_POST['bandeira'] == 'diners' )? 'checked="checked"':'';
				$amex			= (isset( $_POST['bandeira'] ) && $_POST['bandeira'] == 'amex' )? 'checked="checked"':'';
				$elo			= (isset( $_POST['bandeira'] ) && $_POST['bandeira'] == 'elo' )? 'checked="checked"':'';
				$discover		= (isset( $_POST['bandeira'] ) && $_POST['bandeira'] == 'discover' )? 'checked="checked"':'';
				
				
				$janeiro	 = (isset($_POST['mes']) && $_POST['mes'] == '01')?'selected':'';
				$fevereiro	 = (isset($_POST['mes']) && $_POST['mes'] == '02')?'selected':'';
				$marco		 = (isset($_POST['mes']) && $_POST['mes'] == '03')?'selected':'';
				$abril		 = (isset($_POST['mes']) && $_POST['mes'] == '04')?'selected':'';
				$maio		 = (isset($_POST['mes']) && $_POST['mes'] == '05')?'selected':'';
				$junho		 = (isset($_POST['mes']) && $_POST['mes'] == '06')?'selected':'';
				$julho		 = (isset($_POST['mes']) && $_POST['mes'] == '07')?'selected':'';
				$agosto		 = (isset($_POST['mes']) && $_POST['mes'] == '08')?'selected':'';
				$setembro	 = (isset($_POST['mes']) && $_POST['mes'] == '09')?'selected':'';
				$outubro	 = (isset($_POST['mes']) && $_POST['mes'] == '10')?'selected':'';
				$novembro	 = (isset($_POST['mes']) && $_POST['mes'] == '11')?'selected':'';
				$dezembro	 = (isset($_POST['mes']) && $_POST['mes'] == '12')?'selected':'';
				
				$cartoes = "";
				
				if( in_array( 'visa', $this->selected_payment_type ) ){
					$cartoes .= '
				    	<label class="cartoes">
						    <input type="radio" name="bandeira" id="visa" value="visa" '. $visa .' >
					    	<img src="'. $imagens .'/cartao_visa.png" border="0" alt="Visa" />
				    	</label>';
				}
				
				if( in_array( 'mastercard', $this->selected_payment_type ) ){
					$cartoes .= '
				    	<label class="cartoes">
						    <input type="radio" name="bandeira" id="mastercard" value="mastercard" '. $mastercard .' >
						    <img src="'. $imagens .'/cartao_mastercard.png" border="0" alt="mastercard" />
					    </label>';
				}
				
				if( in_array( 'diners', $this->selected_payment_type ) ){
					$cartoes .= '
				    	<label class="cartoes">
						    <input type="radio" name="bandeira" id="diners" value="diners" '. $diners .' >
						    <img src="'. $imagens .'/cartao_diners.png" border="0" alt="Diners" />
					    </label>';
				}
				
				if( in_array( 'amex', $this->selected_payment_type ) ){
					$cartoes .= '
				    	<label class="cartoes">
						    <input type="radio" name="bandeira" id="amex" value="amex" '. $amex .' >
						    <img src="'. $imagens .'/cartao_amex.png" border="0" alt="Amex" />
					    </label>';
				}
				
				if( in_array( 'elo', $this->selected_payment_type ) ){
					$cartoes .= '
				    	<label class="cartoes">
						    <input type="radio" name="bandeira" id="elo" value="elo" '. $elo .' >
						    <img src="'. $imagens .'/cartao_elo.png" border="0" alt="Elo" />
					    </label>';
				}
				if( in_array( 'discover', $this->selected_payment_type ) ){
					$cartoes .= '
				    	<label class="cartoes">
						    <input type="radio" name="bandeira" id="discover" value="discover" '. $discover .' >
						    <img src="'. $imagens .'/cartao_discover.png" border="0" alt="Discover" />
					    </label>';
				}
					    
				
				
				$html  = '
				
				<table width="100%" border="0" cellspacing="5" cellpadding="5" class="SuperPay">
				  <tr>
				  	<td colspan="2">
				  		<h4>Informações sobre o portador</h4>
				  	</td>
				  <tr>
				  <tr>
				    <td >
				    	<label for="nome_portador_cartao" class="campos">
				    		Nome do títular
				    	</label>
			    		<input type="text" name="nome_portador_cartao" id="nome_portador_cartao">
				    </td>
				    <td>
				    	<label for="cpf_portador_cartao" class="campos">
				    		CPF do titular
				    	</label>
			    		<input type="text" name="cpf_portador_cartao" id="cpf_portador_cartao">
				    </td>
		    	  </tr>
				  <!-- dados adicionais -->
				  <tr>
					<td>
						<label for="endereco" class="campos">
							Endereço
						</label>
						<input type="text" id="endereco" name="endereco" />
					</td>
					<td>
						<label for="numero" class="campos">
							Numero
						</label>
						<input type="text" id="numero" name="numero" />
					</td>
				  </tr>
				  <tr>
					<td>
						<label for="complemento" class="campos">
							Complemento (casa, apartamento, fundo)
						</label>
						<input type="text" id="complemento" name="complemento" />
					</td>
					<td>
						<label for="bairro" class="campos">
							Bairro
						</label>
						<input type="text" id="bairro" name="bairro" />
					</td>
				  </tr>
				  <tr>
				  	<td colspan="2">
				  		<h4>Informações do cartão</h4>
				  	</td>
				  <tr>
				  <tr>
				    <td colspan="2">
				    	
				    '. $cartoes .' 
					    
				    </td>
				  </tr>
				  <tr>
				    <!-- dados a validar -->
				    <td>
				    	<label>
				    		Número do cartão
				    		<input type="text" name="numero_cartao" id="numero_cartao" value="" class="number" maxlength="19" >
				    	</label>
				    </td>
				    <td>
				    	<label>
				    		Código
				    		<input type="text" name="codigo_seguranca_cartao" id="codigo_seguranca_cartao" value="" size="4" maxlength="4" class="number">
				    	</label>
				    </td>
				  </tr>
				  <tr>
				    <td colspan="2">
					    <select name="mes" id="mes">
					    	<option value="">Selecione</option>
					    	<option value="01">Janeiro</option>
					    	<option value="02">Fevereiro</option>
					    	<option value="03">Março</option>
					    	<option value="04">Abril</option>
					    	<option value="05">Maio</option>
					    	<option value="06">Junho</option>
					    	<option value="07">Julho</option>
					    	<option value="08">Agosto</option>
					    	<option value="09">Setembro</option>
					    	<option value="10">Outubro</option>
					    	<option value="11">Novembro</option>
					    	<option value="12">Dezembro</option>
					    </select>
						
					    <select name="ano" id="ano">
					    	<option value="">Selecione</option>
					    	'. $ano .'
					    </select>
				    </td>
				  </tr>
				  <tr>
				    <td colspan="2">
				    	<label>
				    		Parcelas
					    	<select name="parcelas" id="parcelas">
					    		'. $parcelas .'
					    	</select>
					    </label>
				    </td>
				  </tr>
				</table>
				
				<script>
jQuery(document).ready(function(){

	/* Quando houver digitação em componentes com a classe "number" */
	jQuery(".number").keydown(function(event) {
		
		
		/* Testar as teclas não numérica */
	    if (event.keyCode == 46 || event.keyCode == 8 || event.keyCode == 9 || event.keyCode == 27 || event.keyCode == 13 || (event.keyCode == 65 && event.ctrlKey === true) || (event.keyCode >= 35 && event.keyCode <= 39)){
			return;
	    }else{
	    
	    	/* Testar acionamento de outras teclas */
	        if (event.shiftKey || (event.keyCode < 48 || event.keyCode > 57) && (event.keyCode < 96 || event.keyCode > 105 )){
	            event.preventDefault(); 
	        }
	    }
	});
	
	

/*
	jQuery( "#place_order" ).click( function(){
		contador = 0;
		if( jQuery( "#payment_method_superpay" ).prop( "checked" ) ){
			jQuery( ".SuperPay input" ).each(function(){
				if( jQuery( this ).val() != "" ){
					contador++;
				}
			});
			if( contador >= 10 ){
				setTimeout(function(){jQuery(".SuperPay input").each(function(){jQuery(this).val("");});},20);
			}
		}
		
	});
	*/
});


				</script>
				
				';
				
				return $html;
			}
			
			
			// Inclui o JS e solicita o JQuery
			public function js_superpay(){
				// registrando o script
				wp_enqueue_script( 'superpay-funcoes', $this->plugin_url .'/js/funcoes.js', array( 'jquery' ), '1.3.1', true );
				
				//wp_enqueue_script( 'funcoes', $this->plugin_url .'/js/funcoes.js', array( 'jquery' ), false, true );
			}
			
			
			/**
			 * Carrega o CSS externo automaticamente
			 *
			 */
			public function css_superpay(){
				
				wp_register_style( 'superpay-estilo', $this->plugin_url .'/css/estilo.css', array(), '1' );
				wp_enqueue_style( 'superpay-estilo' );
			}

			
			
		    /**
		     * Isto testa os campos em busca de erros
		     * caso ocorra um erro o pagamento não
		     * é processado
		     *
		     * @return boolean
		     */
		    public function validate_fields(){
		    	global $woocommerce;
		    	
		    	
				
		    	$campo = $_POST;
		    	
		    	
				if($this->debug=='yes') $this->log->add( $this->id, 'POST '. print_r( $campo, true ) );
		    	
		    	
		    	
		    	// testa nome_portador_cartao
		    	if( isset( $campo['nome_portador_cartao'] ) && empty( $campo['nome_portador_cartao'] ) ){
		    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "O nome do portador não foi informado. Informe o nome correto, exatamente como aparece no cartão.", 'WC_SuperPay' ), 'error' );
					
					if($this->debug=='yes') $this->log->add( $this->id, 'Campo nome portador cartao' );
			    	
		    		return false;
		    	}
		    	
		    	
	
		    	if( isset( $campo['cpf_portador_cartao'] ) ){
		    		
			    	$campo['cpf_portador_cartao'] = str_replace( array( ' ', '.', '-' ), '', $_POST['cpf_portador_cartao'] );
			    	
			    	// testa cpf_portador_cartao
			    	if( empty( $campo['cpf_portador_cartao'] ) ){
			    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "O CPF do títular do cartão precisa ser informado.", 'WC_SuperPay' ), 'error' );
						
						if($this->debug=='yes') $this->log->add( $this->id, 'Campo cpf_portador_cartao' );
				    	
			    		return false;
			    	}
			    	
			    	
			    	// testa numero_cartao
			    	if( !ctype_digit( $campo['cpf_portador_cartao'] ) ){
			    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "O campo CPF precisa conter apenas numeros.", 'WC_SuperPay' ), 'error' );
						
						if($this->debug=='yes') $this->log->add( $this->id, 'Campo CPF letras' );
				    	
			    		return false;
			    	}
			    	
			    	
			    	// testa numero_cartao quantidade
			    	if( strlen( $campo['cpf_portador_cartao'] ) != 11 ){
			    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "O CPF informado parece estar incorreto. Verifique.", 'WC_SuperPay' ), 'error' );
						
						if($this->debug=='yes') $this->log->add( $this->id, 'Campo CPF diferente de 11 digitos' );
				    	
			    		return false;
			    	}
		    	}
		    	
		    	
		    	// testa endereco
		    	if( empty( $campo['endereco'] ) ){
		    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "Você precisa informar o endereço do titular do cartão", 'WC_SuperPay' ), 'error' );
		    		return false;
		    	}
		    	
		    	// testa numero
		    	if( empty( $campo['numero'] ) ){
		    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "Você precisa informar o numero da casa do titular do cartão", 'WC_SuperPay' ), 'error' );
		    		return false;
		    	}
		    	
		    	// testa bairro
		    	if( empty( $campo['bairro'] ) ){
		    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "Você precisa informar o bairro do titular do cartão", 'WC_SuperPay' ), 'error' );
		    		return false;
		    	}
		    	
		    	// testa complemento
		    	if( empty( $campo['complemento'] ) ){
		    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "Você precisa informar o complemento do titular do cartão", 'WC_SuperPay' ), 'error' );
		    		return false;
		    	}
		    	
		    	
		    	
		    	// testa bandeira
		    	if( empty( $campo['bandeira'] )	){
		    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "Você precisa selecionar uma bandeira Visa, Mastercard, Amex, Diners, Discover ou American express.", 'WC_SuperPay' ), 'error' );
					
					if($this->debug=='yes') $this->log->add( $this->id, 'Campo bandeira '. $campo['bandeira'] );
			    	
		    		return false;
		    	}
		    	
		    	
		    	
		    	if( isset( $campo['numero_cartao'] ) ){
		    		
			    	// corrigindo possíveis caracteres não permitidos
			    	$campo['numero_cartao'] = str_replace( array( ' ', '.', '-' ), '', $_POST['numero_cartao'] );
			    	
			    	// testa numero_cartao
			    	if( !$campo['numero_cartao'] ){
			    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "Você precisa informar o número do cartão.", 'WC_SuperPay' ), 'error' );
						
						if($this->debug=='yes') $this->log->add( $this->id, 'Campo numero cartao' );
				    	
			    		return false;
			    	}
			    	
			    	
			    	// testa numero_cartao
			    	if( !ctype_digit( $campo['numero_cartao'] ) ){
			    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "O número do cartão deve possuir apenas números.", 'WC_SuperPay' ), 'error' );
						
						if($this->debug=='yes') $this->log->add( $this->id, 'Campo numero cartao letras' );
				    	
			    		return false;
			    	}
			    	
			    	
			    	// testa numero_cartao quantidade
			    	if( strlen( $campo['numero_cartao'] ) < 14 ){
			    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "O número do cartão informado parece estar incorreto. Verifique. ", 'WC_SuperPay' ), 'error' );
						
						if($this->debug=='yes') $this->log->add( $this->id, 'Campo numero cartao diferente de 14 digitos '. strlen( $campo['numero_cartao'] ) );
				    	
			    		return false;
			    	}
		    	}
		    	
		    	
		    	if( isset( $campo['codigo_seguranca_cartao'] ) ){
		    		
			    	// testa codigo_seguranca_cartao
			    	if( !$campo['codigo_seguranca_cartao'] ){
			    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "Você precisa informar o código de segurança, ele está localizado na parte de traz do cartão.", 'WC_SuperPay' ), 'error' );
						
						if($this->debug=='yes') $this->log->add( $this->id, 'Campo codigo de segurança inexistente' );
				    	
			    		return false;
			    	}
			    	
			    	
			    	// testa codigo_seguranca_cartao
			    	if( !ctype_digit( $campo['codigo_seguranca_cartao'] ) ){
			    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "O código de segurança deve possuir apenas números.", 'WC_SuperPay' ), 'error' );
			    		
						
						if($this->debug=='yes') $this->log->add( $this->id, 'Campo codigo de seguranca com letras' );
				    	
			    		return false;
			    	}
			    	
			    	
			    	// testa codigo_seguranca_cartao quantidade
			    	if( strlen( $campo['codigo_seguranca_cartao'] ) < 3 ){
			    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "O código de segurança informado parece estar incorreto. Verifique.", 'WC_SuperPay' ), 'error' );
						
						if($this->debug=='yes') $this->log->add( $this->id, 'Campo codigo de seguranca menor que 3' );
				    	
			    		return false;
			    	}
		    	}
		    	
	
		    	
		    	// testa validade mes
		    	if( empty( $campo['mes'] ) ){
		    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "Você precisa informar o mês de validade do seu cartão", 'WC_SuperPay' ), 'error' );
		    		return false;
		    	}
		    	
		    	// testa validade ano
		    	if( empty( $campo['ano'] ) ){
		    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "Você precisa informar o ano de validade do cartão", 'WC_SuperPay' ), 'error' );
		    		return false;
		    	}
		    	
		    	
		    	
		    	/*
		    	// testa 
		    	if( strlen( $campo['xxxx'] ) < 3 ){
		    		$this->add_error(__( 'Erro com seu pagamento:', 'WC_SuperPay') .' '. __( "xxxx", 'WC_SuperPay') ) );
		    		return false;
		    	}
		    	*/
		    	
		    	
		    	return false;
		    }
		    
		    
		    
		    /**
		     * Return HTTP status 
		     * this help to found cause of error in transations
		     *
		     * @return status of HTTP request
		     */
		    public function http_status(){
		    	return http_response_code();
		    }

		    
		
		

	
			/**
			 * Add an error message
			 */
			public function add_error( $message, $type = 'error' ) {
				if ( function_exists( 'wc_add_notice' ) ) {
					wc_add_notice( $message, $type );
				} else {
					global $woocommerce;
	
					$woocommerce->add_error( $message );
				}
			}
			
			
		    
		}
	}
	
	function add_superpay_gateway_boleto( $methods ) {
		$methods[] = 'WC_Gateway_SuperPay_Boleto'; 
		return $methods;
	}
	
	
	
}

?>